function [T,Y] = ABM(alpha,fdefun,t0,tfinal,y0, h)
% NOT CURRENTLY FUNCTIONING ACCURATELY. USE FDE12 FOR ACCURATE ABM
% IMPLEMENTATION.
%ABM      Solves an initial value problem for a non-linear differential
%         equation of fractional order (FDE). The code implements the
%         Adams-Bashforth-Moulton method.
%
%   [T,Y] = ABM(ALPHA,FDEFUN,T0,TFINAL,Y0,h) integrates the initial value
%   problem for the FDE, or the system of FDEs, of order ALPHA > 0
%      D^ALPHA Y(t) = FDEFUN(T,Y(T))
%      Y^(k)(0) = y0, k=0,...,m-1
%   where m is the smallest integer greater than ALPHA and D^ALPHA is the 
%   fractional derivative according to the Caputo's definition.
%INPUTS:
%   (1) alpha: The order of the differential equation. Alpha must be >0.
%   (2) fdefun: Fractional Differential Equation. Must be a function handle
%   in the form of fdefun = @(T,Y) ....
%   (3) t0: Initial time. t0 must be a scalar
%   (4) tfinal: Final time. tfinal must be a scalar
%   (5) y0: Initial conditions. See below for info on y0*
%   (6) h: step-size. h must be a scalar value > 0.
%OUTPUTS:
%   (1) T: Time. T should be a vector ranging from t0:h:tfinal
%   (2) Y: Solution to the FDE. Y should be the same length as T, as
%   defined above.
%
% *The set of initial conditions y0 is a matrix with a number of rows equal to
%   the size of the problem (hence equal to the number of rows of the
%   output of FDEFUN) and a number of columns depending on ALPHA and given
%   by m.  
%
%% Error Catching
%Verify that the final time is greater than initial time.
if tfinal <= t0
    error('The initial time, t0, must be less than the final time, tfinal.')
end

%Verify alpha is a positive value.
if alpha <= 0
    error('Alpha must be a positive value')
end

%Check for required number of initial conditions
num_init_conds_req = ceil(alpha);
if size(y0,2) < num_init_conds_req
    error('y0 does not contain sufficient initial condition information to solve the problem. y0 should be a row vector of length equal to ceil(alpha). (E.G. if alpha = 1.25, y0 = [y(0), y_prime(0)])')
end

% Check that fdefun is a function handle.
if isa(fdefun, 'function_handle') == 0
    error('fdefun input is not a function handle')
end

% Verify that the grid is compatible with the method.
tfinal_on_grid_check = (tfinal-t0)/h;
if mod(tfinal_on_grid_check, 1) ~= 0
    error('grid does not end on tfinal. This method requires a constant step size. Please reconfigure your t0, tfinal, and h values such that (tfinal-t0)/h is an integer value.')
end

%% Internal Variable Initialization
%Preallocate Variable Arrays for Speed
num_terms = (tfinal-t0)/h;
y_corrector = zeros(1, num_terms);
y_predictor = zeros(1, num_terms);
f_predictor = zeros(1, num_terms);
f_corrector = zeros(1, num_terms);
g = zeros(1, num_terms);
y_lag_final = zeros(1, num_terms);
y_increment = zeros(1, num_terms);
scheme = 'linear';
%% Initialize Counters:
% NOTE: Since the step can be a non-integer value, (i.e. n =
% t0:h:tfinal), we need an integer variable corresponding to this step for
% array indexing purposes. These are defined below:
% 1) jj: This is the indexing variable for lag calculations. It corresponds
%        with the step j, where j is summed from 0 to n-1
% 2) nn: This is the indexing variable for the method. It corresponds with
% the step n, which is the current timestep. 
nn = 1;

f_corrector(1) = feval(fdefun, t0, y0(1));
y_corrector(1) = y0(1);

%Calculate Coefficients for Linear Interp method. Will remain a constant
%value over the full method, so can be calculated prior to the loop.
[B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t0,t0+h, t0+h, alpha);

%% Begin Method for ABM

% n is the current time step. Starts at t0, and increases by h each time.
% This will calculate values up to final time tfinal - h, since each loop 
% solves the y_(n+1) term.
for n = t0:h:tfinal-h
    
    % Calculate initial condition term for the time-step desired
    g(nn+1) = calculate_IC(y0,alpha,n+h);
 
    
    %Predictor Formula:
    
    %lag term incrementor and lag summer reset for each pass of n.
    jj = 1;
    predictor_sum_term = 0;
    
    for j = 0:h:n
        %Calculate heredity term for the predictor, C_Coeff.
        C_Coeff = h^alpha/alpha*((n+h-j)^alpha - (n-j)^alpha);
        
        %Summing over all previous time steps
        predictor_sum_term = predictor_sum_term + C_Coeff*f_corrector(jj);
        jj = jj+1;
    end
    
    %We are now able to calculate the predictor at the new step:
    y_predictor(nn+1) = g(nn+1) + 1/gamma(alpha)*predictor_sum_term;    
    
    %Evaluate the fdefun using this y_predicted value, at the current time
    %step that we are looking to predict at.
    f_predictor(nn+1) = fdefun(n+h, y_predictor(nn+1));
    
    % Corrector Formula (This is the same Same as MF-PCL)
      jj = 1;
      y_lag_sum = 0;
      
      for j = 0:h:n-h
        %Calculate Coefficients for lag term:
        [B_0j_Coeff, B_1j_Coeff] = B_Coefficients(j,j+h, n+h, alpha);
                
        % Summation Portion of the lag term. 
        y_lag_sum = y_lag_sum + B_0j_Coeff*f_corrector(jj) + B_1j_Coeff*f_corrector(jj+1);
        
        %Increment the interior counter
        jj = jj + 1;
      end 
          
    % Lag term for this time step is now complete!
    y_lag_final(nn+1) = 1/gamma(alpha)*y_lag_sum  ;
  
    %Increment term at the new step
    y_increment(nn+1) = 1/gamma(alpha)*(B_0n_Coeff*f_corrector(nn) + B_1n_Coeff*f_predictor(nn+1));

    %Corrector term at the new step
    y_corrector(nn+1) = g(nn+1) + y_lag_final(nn+1) + y_increment(nn+1);
      
    %Solve using corrector y_value guess instead of predicted y_value. To
    %be used in the next prediction stage.
    f_corrector(nn+1) = feval(fdefun, n+h, y_corrector(nn+1));
    
    %increment n incexing variable.
    nn = nn + 1;
     
end 
%returns the final output variables
Y = y_corrector;
T = t0:h:tfinal;
%% END of Method
end
