function [A_0n_Coeff, A_1n_Coeff, A_2n_Coeff] = A_Coefficients(integrand_low,integrand_high,t_low,t_mid,t_up,t_eval,alpha)
%A_Coefficients is a supporting function in the FDE solver packages MF_PCL/MF_PCQ.
%This function computes the coefficients as defined in [1].
%
%This function has additional inputs as compared to B_Coefficients due to
%additional flexibility required by the startup scheme, StartupProcedure.
%
%INPUTS:
% (1) integrand_low:   (scalar) lower bound of integration
% (2) integrand_high:  (scalar) upper bound of integration, t_high>t_low
% (3) t_low:  (scalar) low value used by the quadratic interpolator
% (4) t_mid: (scalar) middle value used by the quadratic interpolator
% (5) t_up: (scalar) upper value used by the quadratic interpolator
% (6) t_eval: current time-step the coefficient is to be evaluated at
% (7) alpha: (scalar) order of the FDE. alpha > 0.
%OUTPUTS:
% (1) A_0n_Coeff: (scalar) Coefficient calculated using Q_low
% (2) A_1n_Coeff: (scalar) Coefficient calculated using Q_mid
% (3) A_2n_Coeff: (scalar) Coefficient calculated using Q_up
%
%%[1] T. B. Nugyen and B. Jang, A high-order predictor-corrector method for
%    solving nonlinear differential equations of fractional order, 
%    Fractional Calculus and Applied Analysis, vol. 20, No. 2, pp. 447476, 2017. 

% Call the quadratic Lagrangian interpolation function
[Q_low, Q_mid, Q_up] = Quadratic_Interp(t_low, t_mid, t_up);

%The core of the integral, this portion is the same for A_0, A_1, and A_2.
A_kernel = @(tau) ((t_eval-tau).^(alpha-1));

%These lines combine the kernel with each appropriate interpolation
%function, combining both function handles into one:
A_0n_funct = @(tau) A_kernel(tau).*Q_low(tau);
A_1n_funct = @(tau) A_kernel(tau).*Q_mid(tau);
A_2n_funct = @(tau) A_kernel(tau).*Q_up(tau);

%These lines compute the integral of the overall function, using the bounds
%input:
A_0n_Coeff = integral(A_0n_funct,integrand_low,integrand_high);
A_1n_Coeff = integral(A_1n_funct,integrand_low,integrand_high);        
A_2n_Coeff = integral(A_2n_funct,integrand_low,integrand_high);
end