function [B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t_j, t_jj, t_nn, alpha)
%B_Coefficients is a supporting function in the FDE solver packages MF_PCL/MF_PCQ.
%This function computes the coefficients as defined in [1].

%INPUTS:
% (1) t_j:   (scalar) lower bound of integration
% (2) t_jj:  (scalar) upper bound of integration, t_jj>t_j
% (3) t_nn:  (scalar) current time-step the coefficient is to be evaluated at
% (4) alpha: (scalar) order of the FDE. alpha > 0.
%OUTPUTS:
% (1) B_0n_Coeff: (scalar) Coefficient calculated using L_j
% (2) B_1n_Coeff: (scalar) Coefficient calculated using L_jj
%
%%[1] T. B. Nugyen and B. Jang, A high-order predictor-corrector method for
%    solving nonlinear differential equations of fractional order, 
%    Fractional Calculus and Applied Analysis, vol. 20, No. 2, pp. 447476, 2017.

% Call the linear Lagrangian interpolation function
[L_j, L_jj] = Linear_Interp(t_j,t_jj);

%The core of the integral, this portion is the same for both B_0 and B_1.
B_kernel = @(tau) ((t_nn-tau).^(alpha-1));

%These lines combine the kernel with each appropriate interpolation
%function, combining both function handles into one:
B_0n_funct = @(tau) B_kernel(tau).*L_j(tau);
B_1n_funct = @(tau) B_kernel(tau).*L_jj(tau);

%These lines compute the integral of the overall function, using the bounds
%input:
B_0n_Coeff = integral(B_0n_funct, t_j,t_jj);
B_1n_Coeff = integral(B_1n_funct, t_j,t_jj);

end