function [T,Y] = MF_PCQ(alpha,fdefun,t0,tfinal,y0,h)
%MF_PCQ   Solves an initial value problem for a non-linear differential
%         equation of fractional order (FDE). The code implements the
%         3rd-order quadratic interpolation predictor-corrector PECE method of [1].
%
%   [T,Y] = MF_PCQ(ALPHA,FDEFUN,T0,TFINAL,Y0,h) integrates the initial value
%   problem for the FDE, or the system of FDEs, of order ALPHA > 0
%      D^ALPHA Y(t) = FDEFUN(T,Y(T))
%      Y^(k)(0) = y0, k=0,...,m-1
%   where m is the smallest integer greater than ALPHA and D^ALPHA is the 
%   fractional derivative according to the Caputo's definition.
%INPUTS:
%   (1) alpha: The order of the differential equation. Alpha must be >0.
%   (2) fdefun: Fractional Differential Equation. Must be a function handle
%   in the form of fdefun = @(T,Y) ....
%   (3) t0: Initial time. t0 must be a scalar
%   (4) tfinal: Final time. tfinal must be a scalar
%   (5) y0: Initial conditions. See below for info on y0*
%   (6) h: step-size. h must be a scalar value > 0.
%OUTPUTS:
%   (1) T: Time. T should be a vector ranging from t0:h:tfinal
%   (2) Y: Solution to the FDE. Y should be the same length as T, as
%   defined above.
%
% *The set of initial conditions y0 is a matrix with a number of rows equal to
%   the size of the problem (hence equal to the number of rows of the
%   output of FDEFUN) and a number of columns depending on ALPHA and given
%   by m.  
%
%[1] T. B. Nugyen and B. Jang, A high-order predictor-corrector method for
%    solving nonlinear differential equations of fractional order, 
%    Fractional Calculus and Applied Analysis, vol. 20, No. 2, pp. 447476, 2017.


%% Error Catching
%Verify that the final time is greater than initial time.
if tfinal <= t0
    error('The initial time, t0, must be less than the final time, tfinal.')
end

%Verify alpha is a positive value.
if alpha <= 0
    error('Alpha must be a positive value')
end

%Check for required number of initial conditions
num_init_conds_req = ceil(alpha);
if size(y0,2) <= num_init_conds_req
    error('y0 does not contain sufficient initial condition information to solve the problem. y0 should be a row vector of length equal to ceil(alpha). (E.G. if alpha = 1.25, y0 = [y(0), y_prime(0)])')
end

% Check that fdefun is a function handle.
if isa(fdefun, 'function_handle') == 0
    error('fdefun input is not a function handle')
end

% Verify that the grid is compatible with the method.
tfinal_on_grid_check = (tfinal-t0)/h;
if mod(tfinal_on_grid_check, 1) ~= 0
    error('grid does not end on tfinal. This method requires a constant step size. Please reconfigure your t0, tfinal, and h values such that (tfinal-t0)/h is an integer value.')
end

%% Internal Variable Initialization
a0 = (alpha + 4)/2;
a1 = -2*(alpha+3);
a2 = (2*alpha^2 + 9*alpha + 12)/2;
m = 1; % TODO: should be calculated later with IC

%% Initialize Counters:
% NOTE: Since the step can be a non-integer value, (i.e. n =
% t0:h:tfinal), we need an integer variable corresponding to this step for
% array indexing purposes. These are defined below:
% 1) jj: This is the indexing variable for lag calculations. It corresponds
%        with the step j, where j is summed from 0 to n-1
% 2) nn: This is the indexing variable for the method. It corresponds with
% the step n, which is the current timestep. 

%Again, since the startup takes us through 2*h, we have the following:
%nn = 1: y(0)
%nn = 2: y(h)
%nn = 3: y(2h), with y(nn+1) being the first unknown term of the algorithm.
nn = 3;
starting_step = 2*h + t0;

% Startup Export contains the values needed from the startupProcedure. As
% defined below, StartupProcedure contains:
% 1) startup_export(1) = f_corrector evaluated at t = h/2
% 2) startup_export(2) = f_corrector evaluated at t = h
% 3) startup_export(3) = f_corrector evaluated at t = 2h
% 4) startup_export(4) = y_corrector evaluated at t = h/2
% 5) startup_export(5) = y_corrector evaluated at t = h
% 6) startup_export(5) = y_corrector evaluated at t = 2h
scheme = 'quadratic';
startup_export = StartupProcedure(fdefun, scheme, t0, y0, alpha, h);
f_corrector(1) = fdefun(t0, y0(1));
f_corrector(2) = startup_export(2);
f_corrector_one_half = startup_export(1);
f_corrector(3) = startup_export(3);

y_corrector(1) = y0(1);
y_corrector(2) = startup_export(5);
y_corrector_one_half = startup_export(4);
y_corrector(3) = startup_export(6);

%Calculate Initial Condition
g(1) =  calculate_IC(y0, alpha, 0);

[A_0n_Coeff, A_1n_Coeff, A_2n_Coeff] = A_Coefficients(starting_step, starting_step+h, starting_step-h, starting_step, starting_step+h, starting_step+h, alpha);


%% 3rd Order, Quadratic Interp

% n is the current time step. Starts at 2*h, and increases by h each time.
% This is because the startup procedure calculates up through 2*h, so the
% next y_(n+1) term desired is y_(2*h + h). % This will calculate values up
% to final time tfinal, since each loop solves the y_(n+1) term.
for n = starting_step:h:tfinal-h
    
      %lag term incrementor and lag summer reset for each pass of n.
    jj = 2;
    y_lag_sum = 0;
      
    for j = h:h:n-h
        %Calculate Coefficients for lag terms:
        [A_0j_Coeff, A_1j_Coeff, A_2j_Coeff] = A_Coefficients(j, j+h,j-h, j, j+h, n+h, alpha);
                
        % Summation Portion of the lag term. 
        y_lag_sum = y_lag_sum + A_0j_Coeff*f_corrector(jj-1) + A_1j_Coeff*f_corrector(jj) + A_2j_Coeff*f_corrector(jj+1);
        
        %Increment the interior counter
        jj = jj + 1;
    end 
      
    %Calculate the initial condition term 
    g(nn+1) = calculate_IC(y0, alpha, n+h);

    
    %Compute the initial set of coefficients, evaluated at n+h.
    [A_00_Coeff, A_10_Coeff, A_20_Coeff] = A_Coefficients(0,h,0,h/2,h,n+h, alpha);
    
    % Apply Coefficient weights to the fde evaluated at respective time
    % steps
    A_initial = A_00_Coeff*f_corrector(1) + A_10_Coeff*f_corrector_one_half + A_20_Coeff*f_corrector(2);
    
    % Lag term for this time step is now complete!
    y_lag_final(nn+1) = 1/gamma(alpha)*(A_initial + y_lag_sum);
    
    
    %Now we are able to calculate the predictor:
    
    % Predictor term at the new step.
    y_predictor(nn+1) = g(nn+1) + y_lag_final(nn+1) + h^alpha/gamma(alpha+3)*(a0*f_corrector(nn-2) + a1*f_corrector(nn-1) + a2*f_corrector(nn));
    
    %Evaluate the FDEfun using this y_predicted value, at the current time
    %step we are looking to predict at.
    f_predictor(nn+1) = fdefun(n+h, y_predictor(nn+1));
    
    %Increment term at the new step
    y_increment(nn+1) = 1/gamma(alpha)*(A_0n_Coeff*f_corrector(nn-1) + A_1n_Coeff*f_corrector(nn) + A_2n_Coeff*f_predictor(nn+1));

    %Corrector term at the new step, using the same lag term, and  above
    %increment term
    y_corrector(nn+1) = g(nn+1) + y_lag_final(nn+1) + y_increment(nn+1);
      
    %Evaluate using corrector y_value guess instead of predicted y_value. To
    %be used in the next prediction stage.
    f_corrector(nn+1) = feval(fdefun, n+h, y_corrector(nn+1));
    
    %increment n indexing variable
    nn = nn + 1;
     
end 

%return the final output variables 
Y = y_corrector;
T = t0:h:tfinal;
%% END of Method
end
