function [startup_export] = StartupProcedure(fdefun, scheme, t0, y0, alpha, h)
% StartupProcedure is a supporting function to the FDE solver packages
% MF_PCL/MF_PCQ. This function is called by both schemes to begin the
% method. It uses a combination of constant, linear, and quadratic
% interpolation to provide a high-accuracy early-heredity by computing
% values over a non-constant-sized grid of [0, h/4, h/2, h, 2*h]. This
% function follows the definition as described in [1].
%
%INPUTS: 
% (1) fdefun: (function handle @(T,Y) ...) The fractional differential equation.
% (2) scheme: (string) 'linear' or 'quadratic'. This determines which
%             values are returned from the function call, depending on which 
%             scheme is being used.
% (3) t0:     (scalar) initial time. t0 must be >= 0.
% (4) y0:     (vector) The initial value of the problem.
% (5) alpha:  (scalar) order of the FDE. alpha > 0.
% (6) h:      (scalar) This is the step-size of the problem. The startup
%             procedure will divide h into the grid described above. h >0.
%OUTPUTS:
% (1) startup_export: (vector) % array that exports the required values 
%                      computed in the startup procedure:
%                      Linear scheme: [f_corrector(h), f_corrector(2h),
%                                      y_corrector(h), y_corrector_(2h)] 
%                      Quadratic scheme: [f_corrector(h/2), f_corrector(h),
%                                         f_corrector(2h), y_corrector(h/2),
%                                         y_corrector(h), y_corrector_(2h)]
%                      
% %[1] T. B. Nugyen and B. Jang, A high-order predictor-corrector method for
%    solving nonlinear differential equations of fractional order, 
%    Fractional Calculus and Applied Analysis, vol. 20, No. 2, pp. 447476, 2017.


%% Initial Values
g = zeros(1,10);
%Preallocate Variable arrays
f_predictor = zeros(1, 5);
y_predictor = zeros(1, 5);
f_corrector = zeros(1,5);
y_corrector = zeros(1,5);
y_predictor_P1 = zeros(1,5);
y_predictor_P2 = zeros(1,5);
f_predictor_P1 = zeros(1,5);
f_predictor_P2 = zeros(1,5);


f_corrector(1) = fdefun( t0, y0(1));

%% Approximate y(h/4) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Predict y(h/4):
jj = 1;
g(jj+1) = calculate_IC(y0, alpha, t0+h/4);
y_predictor(jj+1) = g(jj+1) + 1/gamma(alpha + 1)*(h/4)^alpha*f_corrector(jj);

%% Correct y(h/4)
%Intermediate Steps:
% 1. Calculate B coefficients
[B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t0 + 0, t0 + h/4, t0 + h/4, alpha) ;
% 2. Calculate f_pred, using y_pred.
f_predictor(jj+1) = fdefun(t0 + h/4, y_predictor(jj+1)) ;

y_corrector(jj+1) = g(jj+1) + 1/gamma(alpha)*(B_0n_Coeff*f_corrector(jj) + B_1n_Coeff*f_predictor(jj+1));

% Calculate f_corrector(h/4)
f_corrector(jj+1) = fdefun( t0 + h/4, y_corrector(jj+1));

jj = jj + 1;

%% Approximate y(h/2) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next Step Indexing Key:
% jj + 1 = 3, corresponding to h/2.
%     jj = 2, corresponding to h/4.
% jj - 1 = 1, corresponding to 0.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Lag term:
%Intermediate Steps:
%1. Calculate B Coefficients
[B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t0 + 0, t0 + h/4, t0 + h/2, alpha) ;

g(jj+1) = calculate_IC(y0, alpha, t0+h/2);

y_lag(jj+1) = 1/gamma(alpha)*(B_0n_Coeff*f_corrector(jj-1) + B_1n_Coeff*f_corrector(jj));
%% Predict y(h/2) Constant Interp:
y_predictor_P1(jj+1) = g(jj+1) + y_lag(jj+1) + 1/gamma(alpha+1)*(h/4)^alpha*f_corrector(jj);

%% Predict y(h/2) Linear Interp:
%Intermediate Steps:
[B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t0 + h/4, t0 + h/2, t0 + h/2, alpha) ;

f_predictor_P1(jj+1) = fdefun(t0 + h/2, y_predictor_P1(jj+1));

y_predictor_P2(jj+1) = g(jj+1) + y_lag(jj+1) + 1/gamma(alpha)*(B_0n_Coeff*f_corrector(jj) + B_1n_Coeff*f_predictor_P1(jj+1));

%% Correct y(h/2) : Quadratic Interp:
%Intermediate Steps:
[A_0n_Coeff, A_1n_Coeff, A_2n_Coeff] = A_Coefficients(t0 + 0,t0 + h/2,t0 + 0,t0 + h/4,t0 + h/2,t0 + h/2,alpha);

f_predictor_P2(jj+1) = fdefun( t0 + h/2, y_predictor_P2(jj+1));

y_corrector(jj+1) = g(jj+1) + 1/gamma(alpha)*(A_0n_Coeff*f_corrector(jj-1) + A_1n_Coeff*f_corrector(jj) + A_2n_Coeff*f_predictor_P2(jj+1));

f_corrector(jj+1) = fdefun( t0 + h/2, y_corrector(jj+1));

jj = jj +1;

%% Approximate y(h) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next Step Indexing Key:
% jj + 1 = 4, corresponding to h.
%     jj = 3, corresponding to h/2.
% jj - 1 = 2, corresponding to h/4.
% jj - 2 = 1, corresponding to 0.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Lag term:
[B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t0 + 0, t0 + h/2, t0 + h, alpha) ;

g(jj+1) = calculate_IC(y0, alpha, t0+h);

y_lag(jj+1) = 1/gamma(alpha)*(B_0n_Coeff*f_corrector(jj-2) + B_1n_Coeff*f_corrector(jj));

%% Predict y(h) Constant Interp:
y_predictor_P1(jj+1) = g(jj+1) + y_lag(jj+1) + 1/gamma(alpha+1)*(h/2)^alpha*f_corrector(jj);

%% Predict y(h) Linear Interp:
%Intermediate Steps:
[B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t0 + h/2,t0 + h,t0 + h, alpha) ;

f_predictor_P1(jj+1) = fdefun(t0 + h, y_predictor_P1(jj+1));


y_predictor_P2(jj+1) = g(jj+1) + y_lag(jj+1) + 1/gamma(alpha)*(B_0n_Coeff*f_corrector(jj) + B_1n_Coeff*f_predictor_P1(jj+1));

%% Correct y(h) : Quadratic Interp:
%Intermediate Steps:
[A_0n_Coeff, A_1n_Coeff, A_2n_Coeff] = A_Coefficients(t0 + 0,t0 + h,t0 + 0,t0 + h/2,t0 + h,t0 + h,alpha);

f_predictor_P2(jj+1) = fdefun( t0 + h, y_predictor_P2(jj+1));


y_corrector(jj+1) = g(jj+1) + 1/gamma(alpha)*(A_0n_Coeff*f_corrector(jj-2) + A_1n_Coeff*f_corrector(jj) + A_2n_Coeff*f_predictor_P2(jj+1));

f_corrector(jj+1) = fdefun(t0 + h, y_corrector(jj+1));

jj = jj +1;
%% Approximate y(2h) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next Step Indexing Key:
% jj + 1 = 5, corresponding to 2h.
%     jj = 4, corresponding to h.
% jj - 1 = 3, corresponding to h/2.
% jj - 2 = 2, corresponding to h/4.
% jj - 3 = 1, corresponding to 0.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Lag term:
[A_0n_Coeff, A_1n_Coeff, A_2n_Coeff] = A_Coefficients(t0 +0,t0 + h,t0 + 0, t0 + h/2,t0 + h,t0 + 2*h, alpha) ;

g(jj+1) = calculate_IC(y0, alpha, t0+2*h);

y_lag(jj+1) = 1/gamma(alpha)*(A_0n_Coeff*f_corrector(jj-3) + A_1n_Coeff*f_corrector(jj-1) + A_2n_Coeff*f_corrector(jj));

%% Predict y(2h) Constant Interp:
y_predictor_P1(jj+1) = g(jj+1) + y_lag(jj+1) + 1/gamma(alpha+1)*(h)^alpha*f_corrector(jj);

%% Predict y(2h) Linear Interp:
%Intermediate Steps:
[B_0n_Coeff, B_1n_Coeff] = B_Coefficients(t0 + h, t0 + 2*h, t0 + 2*h, alpha) ;

f_predictor_P1(jj+1) = fdefun( t0 + 2*h, y_predictor_P1(jj+1));


y_predictor_P2(jj+1) = g(jj+1) + y_lag(jj+1) + 1/gamma(alpha)*(B_0n_Coeff*f_corrector(jj) + B_1n_Coeff*f_predictor_P1(jj+1));

%% Correct y(2h) : Quadratic Interp:
%Intermediate Steps:
[A_0n_Coeff, A_1n_Coeff, A_2n_Coeff] = A_Coefficients(t0 + h, t0 + 2*h, t0 + 0, t0 + h,t0 + 2*h, t0 + 2*h,alpha);

f_predictor_P2(jj+1) = fdefun( t0 + 2*h, y_predictor_P2(jj+1));


y_corrector(jj+1) = g(jj+1) + y_lag(jj+1) + 1/gamma(alpha)*(A_0n_Coeff*f_corrector(jj-3) + A_1n_Coeff*f_corrector(jj) + A_2n_Coeff*f_predictor_P2(jj+1));

f_corrector(jj+1) = fdefun( t0 + 2*h, y_corrector(jj+1));

% Linear vs Quadratic Startup:
%Linear scheme requires values at h and 2h, quadratic scheme requires
%values at h/2, h, and 2h.
if strcmp(scheme, 'linear') == true
    startup_export = [f_corrector(jj), f_corrector(jj+1), y_corrector(jj), y_corrector(jj+1)];
elseif strcmp(scheme, 'quadratic') == true
    startup_export = [f_corrector(jj-1), f_corrector(jj), f_corrector(jj+1), y_corrector(jj-1),  y_corrector(jj), y_corrector(jj+1)];
else
    error('Please enter a valid scheme for the startup algorithm to export the required values. The accepted inputs are "linear", "ABM" and "quadratic".')
end
end

